package aktrs.test.local;

import java.math.BigInteger;
import java.util.logging.Level;

import aktrs.messages.AverageProcessingTimeByClassRequest;
import aktrs.messages.AverageProcessingTimeByClassResponse;
import aktrs.messages.ProfilingMode;
import aktrs.messages.ToggleProfilingMessage;
import aktrs.rt.Actor;
import aktrs.rt.ActorSystem;

/**
 * Profiling actor test
 * 
 * @author Raffaele Arecchi
 *
 */

public class Test3 {

	public static void main(String[] args) {
		ActorSystem sys = new ActorSystem("fib");
		sys.setLogLevel(Level.FINE);

		Actor supervisor = sys.summon("/fib/fibonacciSupervisor", Supervisor.class);
		Actor fib1 = sys.summon("/fib/fibonacciCalc1", FibonacciActor.class);
		supervisor.tell(fib1, ToggleProfilingMessage.create(ProfilingMode.ON));
		supervisor.tell(fib1, CalculateFibonacciMessage.create(new BigInteger("38")));
		supervisor.tell(fib1, CalculateFibonacciMessage.create(new BigInteger("39")));
		AverageProcessingTimeByClassRequest profilingmessage = new AverageProcessingTimeByClassRequest();
		profilingmessage.klass = CalculateFibonacciMessage.class;
		profilingmessage.replyTo = supervisor.getAddress();
		supervisor.tell(fib1, profilingmessage);
	}

	public static class Supervisor extends Actor {

		public Supervisor() {
		}

		@Override
		protected void onMessage(Object message, Actor sender) {
			if (message instanceof AverageProcessingTimeByClassResponse) {
				AverageProcessingTimeByClassResponse m = (AverageProcessingTimeByClassResponse) message;
				System.out.println("Average processing time for klass " + m.klass + " is " + m.averageProcessingTime);
				getSystem().shutdown("end of profiling");
			}
		}
	}

}
