package aktrs.test.local;

import java.io.Serializable;
import java.util.logging.Level;

import aktrs.rt.Actor;
import aktrs.rt.ActorAddress;
import aktrs.rt.ActorSystem;

/**
 * Circular actors communication test
 * 
 * @author Raffaele Arecchi
 *
 */

public class Test5 {

	public static void main(String[] args) {
		ActorSystem sys = new ActorSystem("circularsys");
		sys.setLogLevel(Level.FINE);

		Actor first = sys.summon("/circularsys/circular-actor-1", MemberOfCircleActor.class);
		Actor second = sys.summon("/circularsys/circular-actor-2", MemberOfCircleActor.class);
		Actor third = sys.summon("/circularsys/circular-actor-3", MemberOfCircleActor.class);
		Actor fourth = sys.summon("/circularsys/circular-actor-4", MemberOfCircleActor.class);

		// assignments
		sys.evoke("/circularsys/system").tell(first, AssignNextMessage.create(second.getAddress()));
		sys.evoke("/circularsys/system").tell(second, AssignNextMessage.create(third.getAddress()));
		sys.evoke("/circularsys/system").tell(third, AssignNextMessage.create(fourth.getAddress()));
		sys.evoke("/circularsys/system").tell(fourth, AssignNextMessage.create(first.getAddress()));

		// run
		sys.evoke("/circularsys/system").tell(first, CircularMessage.create("Start!"));
	}

	public static class MemberOfCircleActor extends Actor {

		private Actor nextActor = null;

		public MemberOfCircleActor() {
		}

		@Override
		protected void onMessage(Object message, Actor sender) {
			if (message instanceof AssignNextMessage) {
				AssignNextMessage m = (AssignNextMessage) message;
				nextActor = getSystem().evoke(m.next);
			} else if (message instanceof CircularMessage) {
				CircularMessage m = (CircularMessage) message;
				if (m.message.contains(address.toString())) {
					System.out.println("Circular completed. Final message: " + m.message);
					getSystem().shutdown("end of test");
				} else {
					tell(nextActor, CircularMessage.create(m.message + "\nHello from " + address.toString()));
				}
			}
		}
	}

	public static class CircularMessage implements Serializable {
		private static final long serialVersionUID = 1L;
		public String message;

		public CircularMessage() {
		}

		public static CircularMessage create(String message) {
			CircularMessage m = new CircularMessage();
			m.message = message;
			return m;
		}
	}

	public static class AssignNextMessage implements Serializable {
		private static final long serialVersionUID = 1L;
		public ActorAddress next;

		public AssignNextMessage() {
		}

		public static AssignNextMessage create(ActorAddress next) {
			AssignNextMessage m = new AssignNextMessage();
			m.next = next;
			return m;
		}
	}

}
