package aktrs.test.remote;

import java.io.Serializable;
import java.math.BigInteger;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.logging.Level;

import aktrs.rt.Actor;
import aktrs.rt.ActorAddress;
import aktrs.rt.ActorSystem;

public class Test2_Remote1 {

	public static void main(String[] args) throws UnknownHostException {
		ActorSystem system = new ActorSystem("requester-sys", 9001);
		system.setLogLevel(Level.FINE);
		system.link("fastcompute-sys", InetAddress.getLocalHost(), 9002);

		Actor calculator = system.summon("/fastcompute-sys/calculator", CalculatorActor.class);
		Actor requesterActor = system.summon("/requester-sys/requester", RequesterActor.class);
		system.evoke("/requester-sys/system").tell(requesterActor, AssignCalculator.create(calculator.getAddress()));
		system.evoke("/requester-sys/system").tell(requesterActor, "start");
	}

	public static class RequesterActor extends Actor {
		private Actor calculator;

		public RequesterActor() {
		}

		@Override
		protected void onMessage(Object message, Actor sender) {

			if (message instanceof AssignCalculator) {
				calculator = getSystem().evoke(((AssignCalculator) message).next);
			} else if (message instanceof String && ((String) message).equals("start")) {
				for (int i = 0; i <= 100; i++) {
					for (int j = 0; j < 20; j++) {
						tell(calculator, ComputeMessage.create(j));
					}
				}
			}
		}
	}

	public static class CalculatorActor extends Actor {

		public CalculatorActor() {
		}

		@Override
		protected void onMessage(Object message, Actor sender) {
			if (message instanceof ComputeMessage) {
				ComputeMessage m = (ComputeMessage) message;
				System.out.println("Computing factorial of " + m.in);
				System.out.println("result is " + getFactorialIteratively(new BigInteger("" + m.in)));
			}
		}

		public BigInteger getFactorialIteratively(BigInteger input) {
			if (input.compareTo(BigInteger.ZERO) <= 0) {
				throw new IllegalArgumentException("zero or negatives are not allowed");
			}

			BigInteger result = BigInteger.ONE;
			for (BigInteger i = BigInteger.ONE; i.compareTo(input) <= 0; i = i.add(BigInteger.ONE)) {
				result = result.multiply(i);
			}
			return result;
		}
	}

	public static class AssignCalculator implements Serializable {
		private static final long serialVersionUID = 1L;
		public ActorAddress next;

		public AssignCalculator() {
		}

		public static AssignCalculator create(ActorAddress next) {
			AssignCalculator m = new AssignCalculator();
			m.next = next;
			return m;
		}
	}

	public static class ComputeMessage implements Serializable {
		private static final long serialVersionUID = 1L;
		public Integer in;

		public ComputeMessage() {
		}

		public static ComputeMessage create(Integer in) {
			ComputeMessage m = new ComputeMessage();
			m.in = in;
			return m;
		}
	}

}
